@@ -351,7 +351,13 @@ an `xelb-auto-padding' attribute."
                         (if expression
                             (setq value (xelb-parse-expression expression))
                           (setq value (1+ value)))
-                        `(defconst ,name ,value))))
+			;; Omit the rare enums that do not fit in a fixnum in
+			;; 32-bit Emacs, so that the resulting .el and .elc
+			;; files are portable to 32-bit Emacs.  Admittedly
+			;; this is a kludge.
+			(unless (and (integerp value)
+				     (not (<= -536870912 value 536870911)))
+			  `(defconst ,name ,value)))))
                   items))))
 
 (defun xelb-parse-typedef (node)
@@ -507,12 +513,13 @@ The `combine-adjacent' attribute is simply ignored."
   "Parse a structure content node NODE."
   (pcase (xelb-node-name node)
     (`pad (xelb-parse-pad node))
+    (`required_start_align (xelb-parse-required_start_align node))
     (`field (xelb-parse-field node))
     (`fd (xelb-parse-fd node))
     (`list (xelb-parse-list node))
     (`exprfield (xelb-parse-exprfield node))
     (`switch (xelb-parse-switch node))
-    ((or `comment `doc `required_start_align)) ;simply ignored
+    ((or `comment `doc))                ;simply ignored
     (x (error "Unsupported structure content: <%s>" x))))
 
 ;; The car of the result shall be renamed to prevent duplication of slot names
@@ -528,6 +535,17 @@ The `combine-adjacent' attribute is simply ignored."
              :initform ,(string-to-number align) :type xcb:-pad-align))
         (error "Invalid <pad> field")))))
 
+(defun xelb-parse-required_start_align (node)
+  "Parse <required_start_align>."
+  (let ((align (xelb-node-attr node 'align))
+        (offset (xelb-node-attr node 'offset)))
+    `((,(xelb-generate-pad-name)
+       :initform ,(if offset
+                      (vector (string-to-number align)
+                              (string-to-number offset))
+                    (string-to-number align))
+       :type xcb:-pad-align))))
+
 (defun xelb-parse-field (node)
   "Parse <field>."
   (let* ((name (intern (xelb-node-attr-escape node 'name)))
@@ -545,16 +563,10 @@ The `combine-adjacent' attribute is simply ignored."
          (name-alt (intern (concat (xelb-node-attr-escape node 'name) "~")))
          (type (xelb-node-type node))
          (size (xelb-parse-expression (xelb-node-subnode node))))
-    `((,name :initarg ,(intern (concat ":" (symbol-name name)))
-             :type xcb:-ignore)
-      (,name-alt :initform '(name ,name type ,type size ,size)
+    `((,name-alt :initform '(name ,name type ,type size ,size)
                  :type xcb:-list)
-      ;; Auto padding after variable-length list
-      ;; FIXME: according to the definition of `XCB_TYPE_PAD' in xcb.h, it does
-      ;;        not always padding to 4 bytes.
-      ,@(when (and (xelb-node-attr node 'xelb-auto-padding)
-                   (not (integerp size)))
-          `((,(xelb-generate-pad-name) :initform 4 :type xcb:-pad-align))))))
+      (,name :initarg ,(intern (concat ":" (symbol-name name)))
+             :type xcb:-ignore))))
 
 ;; The car of result is the field declaration, and the cadr is the expression
 ;; to be evaluated.
@@ -600,7 +612,6 @@ The `combine-adjacent' attribute is simply ignored."
                       (when (or (eq case-name 'bitcase) (eq case-name 'case))
                         (dolist (j (xelb-node-subnodes i t))
                           (pcase (xelb-node-name j)
-                            (`required_start_align)
                             (`enumref
                              (setq condition
                                    (nconc condition
@@ -610,10 +621,7 @@ The `combine-adjacent' attribute is simply ignored."
                              (setq fields (nconc fields tmp))
                              (setq name-list
                                    (nconc name-list (list (caar tmp)))))))
-                        (if (eq case-name 'case)
-                            (when (= 1 (length condition))
-                              ;; Flatten 1-element list.
-                              (setq condition (car condition)))
+                        (when (eq case-name 'bitcase)
                           (setq condition (if (= 1 (length condition))
                                               ;; Flatten 1-element list.
                                               (car condition)
