@@ -52,6 +52,14 @@
 (require 'cl-generic)
 (require 'eieio)
 
+(eval-when-compile
+  (defvar xcb:debug-on nil "Non-nil to turn on debug."))
+
+(defmacro xcb:-log (format-string &rest args)
+  "Print debug info."
+  (when xcb:debug-on
+    `(message (concat "[XELB LOG] " ,format-string) ,@args)))
+
 ;;;; Fix backward compatibility issues with Emacs < 25
 
 (eval-and-compile
@@ -155,8 +163,8 @@
                       (logand (lsh value -16) #xFF)
                       (logand (lsh value -8) #xFF)
                       (logand value #xFF))
-            (let* ((msdw (min #xFFFFFFFF (truncate value #x100000000)))
-                   (lsdw (min #xFFFFFFFF
+            (let* ((msdw (min 4294967295. (truncate value 4294967296.)))
+                   (lsdw (min 4294967295.
                               (truncate (- value (* msdw 4294967296.0))))))
               (vector (logand (lsh msdw -24) #xFF) (logand (lsh msdw -16) #xFF)
                       (logand (lsh msdw -8) #xFF) (logand msdw #xFF)
@@ -173,8 +181,8 @@
                       (logand (lsh value -40) #xFF)
                       (logand (lsh value -48) #xFF)
                       (logand (lsh value -56) #xFF))
-            (let* ((msdw (min #xFFFFFFFF (truncate value #x100000000)))
-                   (lsdw (min #xFFFFFFFF
+            (let* ((msdw (min 4294967295. (truncate value 4294967296.)))
+                   (lsdw (min 4294967295.
                               (truncate (- value (* msdw 4294967296.0))))))
               (vector (logand lsdw #xFF) (logand (lsh lsdw -8) #xFF)
                       (logand (lsh lsdw -16) #xFF) (logand (lsh lsdw -24) #xFF)
@@ -207,11 +215,11 @@
           (vector 0 0 0 0
                   (logand (lsh value -24) #xFF) (logand (lsh value -16) #xFF)
                   (logand (lsh value -8) #xFF) (logand value #xFF))
-        (let* ((msw (min #xFFFF (truncate value #x1000000000000)))
+        (let* ((msw (min #xFFFF (truncate value 281474976710656.)))
                (w1 (min #xFFFF
                         (truncate (setq value
                                         (- value (* msw 281474976710656.0)))
-                                  #x100000000)))
+                                  4294967296.)))
                (w2 (min #xFFFF
                         (truncate (setq value (- value (* w1 4294967296.0)))
                                   #x10000)))
@@ -226,11 +234,11 @@
           (vector (logand value #xFF) (logand (lsh value -8) #xFF)
                   (logand (lsh value -16) #xFF) (logand (lsh value -24) #xFF)
                   0 0 0 0)
-        (let* ((msw (min #xFFFF (truncate value #x1000000000000)))
+        (let* ((msw (min #xFFFF (truncate value 281474976710656.)))
                (w1 (min #xFFFF
                         (truncate (setq value
                                         (- value (* msw 281474976710656.0)))
-                                  #x100000000)))
+                                  4294967296.)))
                (w2 (min #xFFFF
                         (truncate (setq value (- value (* w1 4294967296.0)))
                                   #x10000)))
@@ -244,13 +252,13 @@
   "4 bytes signed integer => byte array (MSB first)."
   (xcb:-pack-u4 (if (>= value 0)
                     value
-                  (+ value #x100000000)))) ;treated as float for 32-bit
+                  (+ value 4294967296.)))) ;treated as float for 32-bit
 
 (defsubst xcb:-pack-i4-lsb (value)
   "4 bytes signed integer => byte array (LSB first)."
   (xcb:-pack-u4-lsb (if (>= value 0)
                         value
-                      (+ value #x100000000)))) ;treated as float for 32-bit
+                      (+ value 4294967296.)))) ;treated as float for 32-bit
 
 (defsubst xcb:-unpack-u1 (data offset)
   "Byte array => 1 byte unsigned integer."
@@ -361,16 +369,16 @@
 (defsubst xcb:-unpack-i4 (data offset)
   "Byte array => 4 bytes signed integer (MSB first)."
   (let ((value (xcb:-unpack-u4 data offset)))
-    (if (< value #x80000000)            ;treated as float for 32-bit
+    (if (< value 2147483648.)           ;treated as float for 32-bit
         value
-      (- value #x100000000))))          ;treated as float for 32-bit
+      (- value 4294967296.))))          ;treated as float for 32-bit
 
 (defsubst xcb:-unpack-i4-lsb (data offset)
   "Byte array => 4 bytes signed integer (LSB first)."
   (let ((value (xcb:-unpack-u4-lsb data offset)))
-    (if (< value #x80000000)            ;treated as float for 32-bit
+    (if (< value 2147483648.)           ;treated as float for 32-bit
         value
-      (- value #x100000000))))          ;treated as float for 32-bit
+      (- value 4294967296.))))          ;treated as float for 32-bit
 
 (defmacro xcb:-fieldref (field)
   "Evaluate a <fieldref> field."
@@ -442,7 +450,16 @@
 
 Consider let-bind it rather than change its global value."))
 
-(defclass xcb:-struct ()
+(defclass xcb:--struct ()
+  nil)
+
+(cl-defmethod slot-unbound ((object xcb:--struct) class slot-name fn)
+  (xcb:-log "unbount-slot: %s" (list (eieio-class-name class)
+                                     (eieio-object-name object)
+			             slot-name fn))
+  nil)
+
+(defclass xcb:-struct (xcb:--struct)
   ((~lsb :initarg :~lsb
          :initform (symbol-value 'xcb:lsb) ;see `eieio-default-eval-maybe'
          :type xcb:-ignore))
@@ -492,10 +509,14 @@ The optional POS argument indicates current byte index of the field (used by
        (setq value (eval value `((obj . ,obj)))))
      (make-vector value 0))
     (`xcb:-pad-align
-     (unless (integerp value)
-       (setq value (eval value `((obj . ,obj)))))
      ;; The length slot in xcb:-request is left out
      (let ((len (if (object-of-class-p obj 'xcb:-request) (+ pos 2) pos)))
+       (when (vectorp value)
+         ;; Alignment with offset.
+         (setq len (- len (aref value 1))
+               value (aref value 0)))
+       (unless (integerp value)
+         (setq value (eval value `((obj . ,obj)))))
        (make-vector (% (- value (% len value)) value) 0)))
     (`xcb:-list
      (let* ((list-name (plist-get value 'name))
@@ -546,28 +567,33 @@ The optional POS argument indicates current byte index of the field (used by
      (xcb:marshal value))
     (x (error "[XCB] Unsupported type for marshalling: %s" x))))
 
-(cl-defmethod xcb:unmarshal ((obj xcb:-struct) byte-array &optional ctx)
+(cl-defmethod xcb:unmarshal ((obj xcb:-struct) byte-array &optional ctx
+                             total-length)
   "Fill in fields of struct OBJ according to its byte-array representation.
 
 The optional argument CTX is for <paramref>."
+  (unless total-length
+    (setq total-length (length byte-array)))
   (let ((slots (eieio-class-slots (eieio-object-class obj)))
         (result 0)
         slot-name tmp type)
-    (dolist (slot slots)
-      (setq type (cl--slot-descriptor-type slot))
-      (unless (eq type 'xcb:-ignore)
-        (setq slot-name (eieio-slot-descriptor-name slot)
-              tmp (xcb:-unmarshal-field obj type byte-array 0
-                                        (when (slot-boundp obj slot-name)
-                                          (eieio-oref-default obj slot-name))
-                                        ctx))
-        (setf (slot-value obj slot-name) (car tmp))
-        (setq byte-array (substring byte-array (cadr tmp)))
-        (setq result (+ result (cadr tmp)))))
+    (catch 'break
+      (dolist (slot slots)
+        (setq type (cl--slot-descriptor-type slot))
+        (unless (eq type 'xcb:-ignore)
+          (setq slot-name (eieio-slot-descriptor-name slot)
+                tmp (xcb:-unmarshal-field obj type byte-array 0
+                                          (eieio-oref-default obj slot-name)
+                                          ctx total-length))
+          (setf (slot-value obj slot-name) (car tmp))
+          (setq byte-array (substring byte-array (cadr tmp)))
+          (setq result (+ result (cadr tmp)))
+          (when (eq type 'xcb:-switch) ;xcb:-switch always finishes a struct
+            (throw 'break 'nil)))))
     result))
 
 (cl-defmethod xcb:-unmarshal-field ((obj xcb:-struct) type data offset
-                                    initform &optional ctx)
+                                    initform &optional ctx total-length)
   "Return the value of a field in struct OBJ of type TYPE, byte-array
 representation DATA, and default value INITFORM.
 
@@ -606,12 +632,17 @@ and the second the consumed length."
          (setq initform (cadr initform)))
        (setq initform (eval initform `((obj . ,obj) (ctx . ,ctx)))))
      (list initform initform))
-    (`xcb:-pad-align                 ;assume the whole data is aligned
-     (unless (integerp initform)
-       (when (eq 'quote (car initform))
-         (setq initform (cadr initform)))
-       (setq initform (eval initform `((obj . ,obj) (ctx . ,ctx)))))
-     (list initform (% (- (length data) offset) initform)))
+    (`xcb:-pad-align
+     (let ((len (- total-length (- (length data) offset))))
+       (if (vectorp initform)
+           ;; Alignment with offset.
+           (setq len (- len (aref initform 1))
+                 initform (aref initform 0))
+         (unless (integerp initform)
+           (when (eq 'quote (car initform))
+             (setq initform (cadr initform)))
+           (setq initform (eval initform `((obj . ,obj) (ctx . ,ctx))))))
+       (list initform (% (- initform (% len initform)) initform))))
     (`xcb:-list
      (when (eq 'quote (car initform))   ;unquote the form
        (setq initform (cadr initform)))
@@ -637,7 +668,8 @@ and the second the consumed length."
           (let ((count 0)
                 result tmp)
             (dotimes (_ list-size)
-              (setq tmp (xcb:-unmarshal-field obj x data (+ offset count) nil))
+              (setq tmp (xcb:-unmarshal-field obj x data (+ offset count) nil
+                                              nil total-length))
               (setq result (nconc result (list (car tmp))))
               (setq count (+ count (cadr tmp))))
             (setf (slot-value obj list-name) result)
@@ -672,14 +704,16 @@ and the second the consumed length."
                    (throw 'break nil))))
              (unless (eq slot-type 'xcb:-ignore)
                (setq tmp (xcb:-unmarshal-field obj slot-type data offset
-                                               (eieio-oref-default obj name)))
+                                               (eieio-oref-default obj name)
+                                               nil total-length))
                (setf (slot-value obj name) (car tmp))
                (setq count (+ count (cadr tmp)))
                (setq data (substring data (cadr tmp)))))))
        (list initform count)))
     ((and x (guard (child-of-class-p x 'xcb:-struct)))
      (let* ((struct-obj (make-instance x))
-            (tmp (xcb:unmarshal struct-obj (substring data offset) obj)))
+            (tmp (xcb:unmarshal struct-obj (substring data offset) obj
+                                total-length)))
        (list struct-obj tmp)))
     (x (error "[XCB] Unsupported type for unmarshalling: %s" x))))
 
@@ -757,12 +791,12 @@ This result is converted from the first bounded slot."
       (setq slot (pop slots))
       (setq type (cl--slot-descriptor-type slot)
             name (eieio-slot-descriptor-name slot))
-      (unless (or (not (slot-boundp obj name))
+      (unless (or (not (slot-value obj name))
                   (eq type 'xcb:-ignore)
                   ;; Dealing with `xcb:-list' type
                   (and (eq type 'xcb:-list)
-                       (not (slot-boundp obj (plist-get (slot-value obj name)
-                                                        'name)))))
+                       (not (slot-value obj (plist-get (slot-value obj name)
+                                                       'name)))))
         (setq tmp (xcb:-marshal-field obj (cl--slot-descriptor-type slot)
                                       (slot-value obj name)))
         (when (> (length tmp) (length result))
@@ -771,10 +805,13 @@ This result is converted from the first bounded slot."
       (setq result (vconcat result (make-vector (- size (length result)) 0))))
     result))
 ;;
-(cl-defmethod xcb:unmarshal ((obj xcb:-union) byte-array &optional ctx)
+(cl-defmethod xcb:unmarshal ((obj xcb:-union) byte-array &optional ctx
+                             total-length)
   "Fill in every field in union OBJ, according to BYTE-ARRAY.
 
 The optional argument CTX is for <paramref>."
+  (unless total-length
+    (setq total-length (length byte-array)))
   (let ((slots (eieio-class-slots (eieio-object-class obj)))
         slot-name tmp type)
     (dolist (slot slots)
@@ -782,9 +819,8 @@ The optional argument CTX is for <paramref>."
       (unless (eq type 'xcb:-ignore)
         (setq slot-name (eieio-slot-descriptor-name slot)
               tmp (xcb:-unmarshal-field obj type byte-array 0
-                                        (when (slot-boundp obj slot-name)
-                                          (eieio-oref-default obj slot-name))
-                                        ctx))
+                                        (eieio-oref-default obj slot-name)
+                                        ctx total-length))
         (setf (slot-value obj (eieio-slot-descriptor-name slot)) (car tmp))))
     (slot-value obj '~size)))
 
