@@ -59,19 +59,11 @@
 (eval-when-compile (require 'cl-lib))
 (require 'xcb-xproto)
 
-(eval-when-compile
-  (defvar xcb:debug-on nil "Non-nil to turn on debug."))
-
-(defmacro xcb:-log (format-string &rest args)
-  "Print debug info."
-  (when xcb:debug-on
-    `(message (concat "[XELB LOG] " ,format-string) ,@args)))
-
 (defvar xcb:connection-timeout 3 "Connection timeout.")
 
 ;;;; X connection related
 
-(defclass xcb:connection ()
+(defclass xcb:connection (xcb:--struct)
   ((process :initarg :process :initform nil)
    (connected :initform nil)  ;non-nil indicates connected to X server
    (display :initarg :display :initform nil)
@@ -96,7 +88,7 @@
    (extra-plist :initform nil)) ;for storing extra data (e.g. by extensions)
   :documentation "X connection.")
 
-(defclass xcb:auth-info ()
+(defclass xcb:auth-info (xcb:--struct)
   ((name :initarg :name :initform "" :type string)
    (data :initarg :data :initform "" :type string))
   :documentation "X connection authentication info.")
@@ -364,7 +356,10 @@ Concurrency is disabled as it breaks the orders of errors, replies and events."
                   (setq event-length (funcall (if xcb:lsb
                                                   #'xcb:-unpack-u4-lsb
                                                 #'xcb:-unpack-u4)
-                                              cache 4))
+                                              cache 4)
+                        ;; event-length indicates additional words to the
+                        ;; first 32 bytes.
+                        event-length (+ 32 (* 4 event-length)))
                   (when (< (length cache) event-length)
                     ;; Too short.
                     (throw 'break nil))
@@ -380,14 +375,16 @@ Concurrency is disabled as it breaks the orders of errors, replies and events."
                   (setq listener
                         (lax-plist-get (slot-value connection 'event-plist)
                                        (vector (aref cache 1))))))
+               ;; Conventional events are 32 bytes in size.
+               (unless event-length
+                 (setq event-length 32))
                (when listener
                  (with-slots (event-queue) connection
                    (setf event-queue (nconc event-queue
                                             `([,listener
-                                               ,(substring cache 0 32)
+                                               ,(substring cache 0
+                                                           event-length)
                                                ,synthetic])))))
-               (unless event-length
-                 (setq event-length 32))
                (xcb:-log "Event received: %s" (substring cache 0 event-length))
                (setq cache (substring cache event-length)))))))
       (setf (slot-value connection 'lock) nil))
@@ -654,7 +651,7 @@ Otherwise no error will ever be reported."
                     (setq error (make-instance
                                  (xcb:-error-number->class obj (car i))))
                     (xcb:unmarshal error (cdr i))
-                    i)
+                    error)
                   error-data))
     (cl-remf (slot-value obj 'reply-plist) sequence)
     (cl-remf (slot-value obj 'error-plist) sequence)
@@ -777,10 +774,11 @@ Or if it's an XKB event, return (XKB-EVENT-CODE [XKB-CODE])."
       ;; Generic event.
       (setq alist (symbol-value
                    (intern-soft (concat prefix "xge-number-class-alist")))
-            result (plist-get (slot-value obj 'extension-opcode-plist) class))
+            result (plist-get (slot-value obj 'extension-opcode-plist)
+                              (intern-soft (substring prefix 0 -1))))
       ;; Ensure the extension has been initialized.
       (when result
-        (setq result `(35 . [,result ,(cdr (rassq class alist))]))))
+        (setq result `(35 . [,result ,(car (rassq class alist))]))))
      ((string= prefix "xcb:xkb:")
       ;; XKB event.
       (eval-and-compile (require 'xcb-xkb))
